/*!-----------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Version: 0.5.2(d49899a916fd99840f6f9178f2dd06e0e7013646)
 * Released under the MIT license
 * https://github.com/Microsoft/vscode/blob/master/LICENSE.txt
 *-----------------------------------------------------------*/
define("vs/languages/html/common/htmlWorker.nls",{"vs/base/common/keyCodes":["Windows","Control","Shift","Alt","Command","Windows","Ctrl","Shift","Alt","Command","Windows"],"vs/languages/html/common/htmlTags":["The html element represents the root of an HTML document.","The head element represents a collection of metadata for the Document.","The title element represents the document's title or name. Authors should use titles that identify their documents even when they are used out of context, for example in a user's history or bookmarks, or in search results. The document's title is often different from its first heading, since the first heading does not have to stand alone when taken out of context.","The base element allows authors to specify the document base URL for the purposes of resolving relative URLs, and the name of the default browsing context for the purposes of following hyperlinks. The element does not represent any content beyond this information.","The link element allows authors to link their document to other resources.","The meta element represents various kinds of metadata that cannot be expressed using the title, base, link, style, and script elements.","The style element allows authors to embed style information in their documents. The style element is one of several inputs to the styling processing model. The element does not represent content for the user.","The body element represents the content of the document.","The article element represents a complete, or self-contained, composition in a document, page, application, or site and that is, in principle, independently distributable or reusable, e.g. in syndication. This could be a forum post, a magazine or newspaper article, a blog entry, a user-submitted comment, an interactive widget or gadget, or any other independent item of content. Each article should be identified, typically by including a heading (h1–h6 element) as a child of the article element.","The section element represents a generic section of a document or application. A section, in this context, is a thematic grouping of content. Each section should be identified, typically by including a heading ( h1- h6 element) as a child of the section element.","The nav element represents a section of a page that links to other pages or to parts within the page: a section with navigation links.","The aside element represents a section of a page that consists of content that is tangentially related to the content around the aside element, and which could be considered separate from that content. Such sections are often represented as sidebars in printed typography.","The h1 element represents a section heading.","The h2 element represents a section heading.","The h3 element represents a section heading.","The h4 element represents a section heading.","The h5 element represents a section heading.","The h6 element represents a section heading.","The header element represents introductory content for its nearest ancestor sectioning content or sectioning root element. A header typically contains a group of introductory or navigational aids. When the nearest ancestor sectioning content or sectioning root element is the body element, then it applies to the whole page.","The footer element represents a footer for its nearest ancestor sectioning content or sectioning root element. A footer typically contains information about its section such as who wrote it, links to related documents, copyright data, and the like.","The address element represents the contact information for its nearest article or body element ancestor. If that is the body element, then the contact information applies to the document as a whole.","The p element represents a paragraph.","The hr element represents a paragraph-level thematic break, e.g. a scene change in a story, or a transition to another topic within a section of a reference book.","The pre element represents a block of preformatted text, in which structure is represented by typographic conventions rather than by elements.","The blockquote element represents content that is quoted from another source, optionally with a citation which must be within a footer or cite element, and optionally with in-line changes such as annotations and abbreviations.","The ol element represents a list of items, where the items have been intentionally ordered, such that changing the order would change the meaning of the document.","The ul element represents a list of items, where the order of the items is not important — that is, where changing the order would not materially change the meaning of the document.","The li element represents a list item. If its parent element is an ol, ul, or menu element, then the element is an item of the parent element's list, as defined for those elements. Otherwise, the list item has no defined list-related relationship to any other li element.","The dl element represents an association list consisting of zero or more name-value groups (a description list). A name-value group consists of one or more names (dt elements) followed by one or more values (dd elements), ignoring any nodes other than dt and dd elements. Within a single dl element, there should not be more than one dt element for each name.","The dt element represents the term, or name, part of a term-description group in a description list (dl element).","The dd element represents the description, definition, or value, part of a term-description group in a description list (dl element).","The figure element represents some flow content, optionally with a caption, that is self-contained (like a complete sentence) and is typically referenced as a single unit from the main flow of the document.","The figcaption element represents a caption or legend for the rest of the contents of the figcaption element's parent figure element, if any.","The main element represents the main content of the body of a document or application. The main content area consists of content that is directly related to or expands upon the central topic of a document or central functionality of an application.","The div element has no special meaning at all. It represents its children. It can be used with the class, lang, and title attributes to mark up semantics common to a group of consecutive elements.","If the a element has an href attribute, then it represents a hyperlink (a hypertext anchor) labeled by its contents.","The em element represents stress emphasis of its contents.","The strong element represents strong importance, seriousness, or urgency for its contents.","The small element represents side comments such as small print.","The s element represents contents that are no longer accurate or no longer relevant.","The cite element represents a reference to a creative work. It must include the title of the work or the name of the author(person, people or organization) or an URL reference, or a reference in abbreviated form as per the conventions used for the addition of citation metadata.","The q element represents some phrasing content quoted from another source.","The dfn element represents the defining instance of a term. The paragraph, description list group, or section that is the nearest ancestor of the dfn element must also contain the definition(s) for the term given by the dfn element.","The abbr element represents an abbreviation or acronym, optionally with its expansion. The title attribute may be used to provide an expansion of the abbreviation. The attribute, if specified, must contain an expansion of the abbreviation, and nothing else.","The ruby element allows one or more spans of phrasing content to be marked with ruby annotations. Ruby annotations are short runs of text presented alongside base text, primarily used in East Asian typography as a guide for pronunciation or to include other annotations. In Japanese, this form of typography is also known as furigana. Ruby text can appear on either side, and sometimes both sides, of the base text, and it is possible to control its position using CSS. A more complete introduction to ruby can be found in the Use Cases & Exploratory Approaches for Ruby Markup document as well as in CSS Ruby Module Level 1. [RUBY-UC] [CSSRUBY]","The rb element marks the base text component of a ruby annotation. When it is the child of a ruby element, it doesn't represent anything itself, but its parent ruby element uses it as part of determining what it represents.","The rt element marks the ruby text component of a ruby annotation. When it is the child of a ruby element or of an rtc element that is itself the child of a ruby element, it doesn't represent anything itself, but its ancestor ruby element uses it as part of determining what it represents.","The rp element is used to provide fallback text to be shown by user agents that don't support ruby annotations. One widespread convention is to provide parentheses around the ruby text component of a ruby annotation.","The time element represents its contents, along with a machine-readable form of those contents in the datetime attribute. The kind of content is limited to various kinds of dates, times, time-zone offsets, and durations, as described below.","The code element represents a fragment of computer code. This could be an XML element name, a file name, a computer program, or any other string that a computer would recognize.","The var element represents a variable. This could be an actual variable in a mathematical expression or programming context, an identifier representing a constant, a symbol identifying a physical quantity, a function parameter, or just be a term used as a placeholder in prose.","The samp element represents sample or quoted output from another program or computing system.","The kbd element represents user input (typically keyboard input, although it may also be used to represent other input, such as voice commands).","The sub element represents a subscript.","The sup element represents a superscript.","The i element represents a span of text in an alternate voice or mood, or otherwise offset from the normal prose in a manner indicating a different quality of text, such as a taxonomic designation, a technical term, an idiomatic phrase from another language, transliteration, a thought, or a ship name in Western texts.","The b element represents a span of text to which attention is being drawn for utilitarian purposes without conveying any extra importance and with no implication of an alternate voice or mood, such as key words in a document abstract, product names in a review, actionable words in interactive text-driven software, or an article lede.","The u element represents a span of text with an unarticulated, though explicitly rendered, non-textual annotation, such as labeling the text as being a proper name in Chinese text (a Chinese proper name mark), or labeling the text as being misspelt.","The mark element represents a run of text in one document marked or highlighted for reference purposes, due to its relevance in another context. When used in a quotation or other block of text referred to from the prose, it indicates a highlight that was not originally present but which has been added to bring the reader's attention to a part of the text that might not have been considered important by the original author when the block was originally written, but which is now under previously unexpected scrutiny. When used in the main prose of a document, it indicates a part of the document that has been highlighted due to its likely relevance to the user's current activity.","The bdi element represents a span of text that is to be isolated from its surroundings for the purposes of bidirectional text formatting. [BIDI]","The bdo element represents explicit text directionality formatting control for its children. It allows authors to override the Unicode bidirectional algorithm by explicitly specifying a direction override. [BIDI]","The span element doesn't mean anything on its own, but can be useful when used together with the global attributes, e.g. class, lang, or dir. It represents its children.","The br element represents a line break.","The wbr element represents a line break opportunity.","The ins element represents an addition to the document.","The del element represents a removal from the document.","The picture element is a container which provides multiple sources to its contained img element to allow authors to declaratively control or give hints to the user agent about which image resource to use, based on the screen pixel density, viewport size, image format, and other factors. It represents its children.","An img element represents an image.","The iframe element represents a nested browsing context.","The embed element provides an integration point for an external (typically non-HTML) application or interactive content.","The object element can represent an external resource, which, depending on the type of the resource, will either be treated as an image, as a nested browsing context, or as an external resource to be processed by a plugin.","The param element defines parameters for plugins invoked by object elements. It does not represent anything on its own.","A video element is used for playing videos or movies, and audio files with captions.","An audio element represents a sound or audio stream.","The source element allows authors to specify multiple alternative media resources for media elements. It does not represent anything on its own.","The track element allows authors to specify explicit external timed text tracks for media elements. It does not represent anything on its own.","The map element, in conjunction with an img element and any area element descendants, defines an image map. The element represents its children.","The area element represents either a hyperlink with some text and a corresponding area on an image map, or a dead area on an image map.","The table element represents data with more than one dimension, in the form of a table.","The caption element represents the title of the table that is its parent, if it has a parent and that is a table element.","The colgroup element represents a group of one or more columns in the table that is its parent, if it has a parent and that is a table element.","If a col element has a parent and that is a colgroup element that itself has a parent that is a table element, then the col element represents one or more columns in the column group represented by that colgroup.","The tbody element represents a block of rows that consist of a body of data for the parent table element, if the tbody element has a parent and it is a table.","The thead element represents the block of rows that consist of the column labels (headers) for the parent table element, if the thead element has a parent and it is a table.","The tfoot element represents the block of rows that consist of the column summaries (footers) for the parent table element, if the tfoot element has a parent and it is a table.","The tr element represents a row of cells in a table.","The td element represents a data cell in a table.","The th element represents a header cell in a table.","The form element represents a collection of form-associated elements, some of which can represent editable values that can be submitted to a server for processing.","The label element represents a caption in a user interface. The caption can be associated with a specific form control, known as the label element's labeled control, either using the for attribute, or by putting the form control inside the label element itself.","The input element represents a typed data field, usually with a form control to allow the user to edit the data.","The button element represents a button labeled by its contents.","The select element represents a control for selecting amongst a set of options.","The datalist element represents a set of option elements that represent predefined options for other controls. In the rendering, the datalist element represents nothing and it, along with its children, should be hidden.","The optgroup element represents a group of option elements with a common label.","The option element represents an option in a select element or as part of a list of suggestions in a datalist element.","The textarea element represents a multiline plain text edit control for the element's raw value. The contents of the control represent the control's default value.","The output element represents the result of a calculation performed by the application, or the result of a user action.","The progress element represents the completion progress of a task. The progress is either indeterminate, indicating that progress is being made but that it is not clear how much more work remains to be done before the task is complete (e.g. because the task is waiting for a remote host to respond), or the progress is a number in the range zero to a maximum, giving the fraction of work that has so far been completed.","The meter element represents a scalar measurement within a known range, or a fractional value; for example disk usage, the relevance of a query result, or the fraction of a voting population to have selected a particular candidate.","The fieldset element represents a set of form controls optionally grouped under a common name.","The legend element represents a caption for the rest of the contents of the legend element's parent fieldset element, if any.","The details element represents a disclosure widget from which the user can obtain additional information or controls.","The summary element represents a summary, caption, or legend for the rest of the contents of the summary element's parent details element, if any.","The dialog element represents a part of an application that a user interacts with to perform a task, for example a dialog box, inspector, or window.","The script element allows authors to include dynamic script and data blocks in their documents. The element does not represent content for the user.","The noscript element represents nothing if scripting is enabled, and represents its children if scripting is disabled. It is used to present different markup to user agents that support scripting and those that don't support scripting, by affecting how the document is parsed.","The template element is used to declare fragments of HTML that can be cloned and inserted in the document by script.","The canvas element provides scripts with a resolution-dependent bitmap canvas, which can be used for rendering graphs, game graphics, art, or other visual images on the fly.","The checkbox is no different than the HTML checkbox input, except it's styled differently. The checkbox behaves like any AngularJS checkbox.","The ionContent directive provides an easy to use content area that can be configured to use Ionic's custom Scroll View, or the built-in overflow scrolling of the browser.","Child of ionItem",'Adds a fixed footer bar below some content. Can also be a subfooter (higher up) if the "bar-subfooter" class is applied.','Adds a fixed header bar above some content. Can also be a subheader (lower down) if the "bar-subheader" class is applied.',"Child of ionContent or ionScroll. The ionInfiniteScroll directive allows you to call a function whenever the user gets to the bottom of the page or near the bottom of the page.",'ionInput is meant for text type inputs only. Ionic uses an actual <input type="text"> HTML element within the component, with Ionic wrapping to better handle the user experience and interactivity.',"Child of ionList.","The List is a widely used interface element in almost any mobile app, and can include content ranging from basic text all the way to buttons, toggles, icons, and thumbnails.","The Modal is a content pane that can go over the user's main view temporarily. Usually used for making a choice or editing an item.","Child of ionNavBar. Creates a back button inside an ionNavBar. The back button will appear when the user is able to go back in the current navigation stack.","If you have an ionNavView directive, you can also create an <ion-nav-bar>, which will create a topbar that updates as the application state changes.","Child of ionNavView. Use ionNavButtons to set the buttons on your ionNavBar from within an ionView.","Child of ionNavView. The ionNavTitle directive replaces an ionNavBar title text with custom HTML from within an ionView template.","The ionNavView directive is used to render templates in your application. Each template is part of a state. States are usually mapped to a url, and are defined programatically using angular-ui-router.","Child of ionItem. Creates an option button inside a list item, that is visible when the item is swiped to the left by the user.",'A simple container that fits content, with no side effects. Adds the "pane" class to the element.',"The Popover is a view that floats above an app's content. Popovers provide an easy way to present or gather information from the user.","The radio ionRirective is no different than the HTML radio input, except it's styled differently. The ionRadio behaves like AngularJS radio input.","Child of ionContent or ionScroll. Allows you to add pull-to-refresh to a scrollView. Place it as the first child of your ionContent or ionScroll element.","Child of ionItem.","Creates a scrollable container for all content inside.","Child of ionSideMenus. A container for a side menu, sibling to an ionSideMenuContent directive.","Child of ionSideMenus. A container for the main visible content, sibling to one or more ionSideMenu directives.","A container element for side menu(s) and the main content. Allows the left and/or right side menu to be toggled by dragging the main content area side to side.","Child of ionSlideBox. Displays a slide inside of a slidebox.","The Slide Box is a multi-page container where each page can be swiped or dragged between.","The ionSpinner directive provides a variety of animated spinners.","Child of ionTabs. Contains a tab's content. The content only exists while the given tab is selected.",'Powers a multi-tabbed interface with a tab bar and a set of "pages" that can be tabbed through.',"ion-title is a component that sets the title of the ionNavbar","A toggle is an animated switch which binds a given model to a boolean. Allows dragging of the switch's nub. The toggle behaves like any AngularJS checkbox otherwise.","Child of ionNavView. A container for view content and any navigational and header bar information."]});
//# sourceMappingURL=../../../../../min-maps/vs/languages/html/common/htmlWorker.nls.js.map